package sun.rmi.server;

import java.net.SocketException;
import java.net.Socket;
import java.net.InetAddress;
import java.net.SocketAddress;
import java.security.Permissions;
import java.security.CodeSource;
import java.security.cert.Certificate;
import java.security.AllPermission;
import java.security.Permission;
import java.security.Policy;
import sun.security.provider.PolicyFile;
import java.security.PermissionCollection;
import com.sun.rmi.rmid.ExecPermission;
import java.security.AccessControlException;
import com.sun.rmi.rmid.ExecOptionPermission;
import sun.rmi.log.LogHandler;
import java.rmi.ConnectIOException;
import java.rmi.ConnectException;
import java.rmi.NoSuchObjectException;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.rmi.activation.ActivationInstantiator;
import java.rmi.activation.ActivationDesc;
import java.io.ObjectInput;
import java.rmi.MarshalledObject;
import sun.rmi.transport.LiveRef;
import java.rmi.server.ObjID;
import java.rmi.server.RemoteServer;
import java.rmi.AlreadyBoundException;
import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.server.RMIClientSocketFactory;
import sun.rmi.registry.RegistryImpl;
import java.rmi.server.RMIClassLoader;
import java.util.MissingResourceException;
import java.net.ServerSocket;
import java.rmi.activation.ActivationGroup;
import sun.security.action.GetPropertyAction;
import sun.security.action.GetBooleanAction;
import java.util.Date;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.nio.file.Files;
import java.nio.file.attribute.FileAttribute;
import java.nio.channels.ServerSocketChannel;
import java.security.PrivilegedExceptionAction;
import java.nio.channels.Channel;
import java.text.MessageFormat;
import java.io.File;
import java.security.PrivilegedAction;
import java.rmi.RemoteException;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Arrays;
import java.util.ArrayList;
import java.rmi.activation.ActivationGroupDesc;
import java.rmi.activation.UnknownObjectException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.Remote;
import java.rmi.activation.ActivationException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.rmi.activation.UnknownGroupException;
import java.rmi.server.RemoteObject;
import java.rmi.server.RMIServerSocketFactory;
import java.util.concurrent.ConcurrentHashMap;
import java.security.AccessController;
import sun.security.action.GetIntegerAction;
import java.util.ResourceBundle;
import java.rmi.registry.Registry;
import java.rmi.activation.ActivationMonitor;
import java.rmi.activation.ActivationSystem;
import java.rmi.activation.Activator;
import sun.rmi.log.ReliableLog;
import java.rmi.activation.ActivationGroupID;
import java.rmi.activation.ActivationID;
import java.util.Map;
import java.lang.reflect.Method;
import java.io.Serializable;

public class Activation implements Serializable
{
    private static final long serialVersionUID = 2921265612698155191L;
    private static final byte MAJOR_VERSION = 1;
    private static final byte MINOR_VERSION = 0;
    private static Object execPolicy;
    private static Method execPolicyMethod;
    private static boolean debugExec;
    private Map<ActivationID, ActivationGroupID> idTable;
    private Map<ActivationGroupID, Activation.GroupEntry> groupTable;
    private byte majorVersion;
    private byte minorVersion;
    private transient int groupSemaphore;
    private transient int groupCounter;
    private transient ReliableLog log;
    private transient int numUpdates;
    private transient String[] command;
    private static final long groupTimeout;
    private static final int snapshotInterval;
    private static final long execTimeout;
    private static final Object initLock;
    private static boolean initDone;
    private transient Activator activator;
    private transient Activator activatorStub;
    private transient ActivationSystem system;
    private transient ActivationSystem systemStub;
    private transient ActivationMonitor monitor;
    private transient Registry registry;
    private transient volatile boolean shuttingDown;
    private transient volatile Object startupLock;
    private transient Thread shutdownHook;
    private static ResourceBundle resources;
    
    private static int getInt(String theProp, int defaultVal) {
        return (int)AccessController.doPrivileged(new GetIntegerAction(theProp, defaultVal));
    }
    
    private Activation() {
        this.idTable = new ConcurrentHashMap();
        this.groupTable = new ConcurrentHashMap();
        this.majorVersion = 1;
        this.minorVersion = 0;
        this.shuttingDown = false;
    }
    
    private static void startActivation(int n, RMIServerSocketFactory rmiServerSocketFactory, String dirPath, String[] array) throws Exception {
        ReliableLog reliableLog = new ReliableLog(dirPath, new Activation.ActLogHandler());
        ((Activation)reliableLog.recover()).init(n, rmiServerSocketFactory, reliableLog, array);
    }
    
    private void init(int n, RMIServerSocketFactory rmiServerSocketFactory, ReliableLog log, String[] array) throws Exception {
        this.log = log;
        this.numUpdates = 0;
        this.shutdownHook = new Activation.ShutdownHook();
        this.groupSemaphore = getInt("sun.rmi.activation.groupThrottle", 3);
        this.groupCounter = 0;
        Runtime.getRuntime().addShutdownHook(this.shutdownHook);
        ActivationGroupID[] array2 = (ActivationGroupID[])this.groupTable.keySet().toArray(new ActivationGroupID[0]);
        Object startupLock = new Object();
        synchronized (this.startupLock = startupLock) {
            this.activator = new Activation.ActivatorImpl(n, rmiServerSocketFactory);
            this.activatorStub = (Activator)RemoteObject.toStub(this.activator);
            this.system = new Activation.ActivationSystemImpl(n, rmiServerSocketFactory);
            this.systemStub = (ActivationSystem)RemoteObject.toStub(this.system);
            this.monitor = new Activation.ActivationMonitorImpl(n, rmiServerSocketFactory);
            this.initCommand(array);
            this.registry = new Activation.SystemRegistryImpl(n, null, rmiServerSocketFactory, this.systemStub);
            if (rmiServerSocketFactory != null) {
                synchronized (Activation.initLock) {
                    Activation.initDone = true;
                    Activation.initLock.notifyAll();
                }
            }
        }
        this.startupLock = null;
        int length = array2.length;
        while (--length >= 0) {
            try {
                this.getGroupEntry(array2[length]).restartServices();
            }
            catch (UnknownGroupException ex) {
                System.err.println(getTextResource("rmid.restart.group.warning"));
                ex.printStackTrace();
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (!(this.groupTable instanceof ConcurrentHashMap)) {
            this.groupTable = new ConcurrentHashMap(this.groupTable);
        }
        if (!(this.idTable instanceof ConcurrentHashMap)) {
            this.idTable = new ConcurrentHashMap(this.idTable);
        }
    }
    
    private void checkShutdown() throws ActivationException {
        Object startupLock = this.startupLock;
        if (startupLock != null) {
            synchronized (startupLock) {}
        }
        if (this.shuttingDown) {
            throw new ActivationException("activation system shutting down");
        }
        return;
    }
    
    private static void unexport(Remote obj) {
        Label_0000: {
            try {
                while (!UnicastRemoteObject.unexportObject(obj, false)) {
                    Thread.sleep(100L);
                }
            }
            catch (Exception ex) {
                continue;
            }
        }
    }
    
    private ActivationGroupID getGroupID(ActivationID obj) throws UnknownObjectException {
        ActivationGroupID activationGroupID = (ActivationGroupID)this.idTable.get(obj);
        if (activationGroupID != null) {
            return activationGroupID;
        }
        throw new UnknownObjectException("unknown object: " + obj);
    }
    
    private Activation.GroupEntry getGroupEntry(ActivationGroupID activationGroupID, boolean b) throws UnknownGroupException {
        if (activationGroupID.getClass() == ActivationGroupID.class) {
            Activation.GroupEntry groupEntry;
            if (b) {
                groupEntry = (Activation.GroupEntry)this.groupTable.remove(activationGroupID);
            }
            else {
                groupEntry = (Activation.GroupEntry)this.groupTable.get(activationGroupID);
            }
            if (groupEntry != null && !groupEntry.removed) {
                return groupEntry;
            }
        }
        throw new UnknownGroupException("group unknown");
    }
    
    private Activation.GroupEntry getGroupEntry(ActivationGroupID activationGroupID) throws UnknownGroupException {
        return this.getGroupEntry(activationGroupID, false);
    }
    
    private Activation.GroupEntry removeGroupEntry(ActivationGroupID activationGroupID) throws UnknownGroupException {
        return this.getGroupEntry(activationGroupID, true);
    }
    
    private Activation.GroupEntry getGroupEntry(ActivationID activationID) throws UnknownObjectException {
        Activation.GroupEntry groupEntry = (Activation.GroupEntry)this.groupTable.get(this.getGroupID(activationID));
        if (groupEntry != null && !groupEntry.removed) {
            return groupEntry;
        }
        throw new UnknownObjectException("object's group removed");
    }
    
    private String[] activationArgs(ActivationGroupDesc activationGroupDesc) {
        ActivationGroupDesc.CommandEnvironment commandEnvironment = activationGroupDesc.getCommandEnvironment();
        ArrayList<String> list = new ArrayList();
        list.add((commandEnvironment != null && commandEnvironment.getCommandPath() != null) ? commandEnvironment.getCommandPath() : this.command[0]);
        if (commandEnvironment != null && commandEnvironment.getCommandOptions() != null) {
            list.addAll(Arrays.asList(commandEnvironment.getCommandOptions()));
        }
        Properties propertyOverrides = activationGroupDesc.getPropertyOverrides();
        if (propertyOverrides != null) {
            Enumeration<Object> propertyNames = propertyOverrides.propertyNames();
            while (propertyNames.hasMoreElements()) {
                String s = (String)propertyNames.nextElement();
                list.add("-D" + s + "=" + propertyOverrides.getProperty(s));
            }
        }
        for (int i = 1; i < this.command.length; ++i) {
            list.add(this.command[i]);
        }
        String[] array = new String[list.size()];
        System.arraycopy(list.toArray(), 0, array, 0, array.length);
        return array;
    }
    
    private void checkArgs(ActivationGroupDesc activationGroupDesc, String[] activationArgs) throws SecurityException, ActivationException {
        if (Activation.execPolicyMethod != null) {
            if (activationArgs == null) {
                activationArgs = this.activationArgs(activationGroupDesc);
            }
            try {
                Activation.execPolicyMethod.invoke(Activation.execPolicy, new Object[] { activationGroupDesc, activationArgs });
            }
            catch (InvocationTargetException ex) {
                Throwable targetException = ex.getTargetException();
                if ((targetException instanceof SecurityException)) {
                    throw (SecurityException)targetException;
                }
                throw new ActivationException(Activation.execPolicyMethod.getName() + ": unexpected exception", ex);
            }
            catch (Exception ex2) {
                throw new ActivationException(Activation.execPolicyMethod.getName() + ": unexpected exception", ex2);
            }
        }
    }
    
    private void addLogRecord(Activation.LogRecord value) throws ActivationException {
        synchronized (this.log) {
            this.checkShutdown();
            try {
                this.log.update(value, true);
            }
            catch (Exception ex) {
                this.numUpdates = Activation.snapshotInterval;
                System.err.println(getTextResource("rmid.log.update.warning"));
                ex.printStackTrace();
            }
            if (++this.numUpdates < Activation.snapshotInterval) {
                return;
            }
            try {
                this.log.snapshot(this);
                this.numUpdates = 0;
            }
            catch (Exception ex2) {
                System.err.println(getTextResource("rmid.log.snapshot.warning"));
                ex2.printStackTrace();
                try {
                    this.system.shutdown();
                }
                catch (RemoteException ex3) {}
                throw new ActivationException("log snapshot failed", ex2);
            }
        }
    }
    
    private void initCommand(String[] array) {
        this.command = new String[array.length + 2];
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                try {
                    Activation.this.command[0] = System.getProperty("java.home") + File.separator + "bin" + File.separator + "java";
                }
                catch (Exception ex) {
                    System.err.println(Activation.access$1800("rmid.unfound.java.home.property"));
                    Activation.this.command[0] = "java";
                }
                return null;
            }
        });
        System.arraycopy(array, 0, this.command, 1, array.length);
        this.command[this.command.length - 1] = "sun.rmi.server.ActivationGroupInit";
    }
    
    private static void bomb(String str) {
        System.err.println("rmid: " + str);
        System.err.println(MessageFormat.format(getTextResource("rmid.usage"), new Object[] { "rmid" }));
        System.exit(1);
    }
    
    public static void main(String[] array) {
        boolean n = false;
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new SecurityManager());
        }
        try {
            final int n2 = 1098;
            Activation.ActivationServerSocketFactory activationServerSocketFactory = null;
            Channel obj = (Channel)AccessController.doPrivileged(new PrivilegedExceptionAction<Channel>() {
                @Override
                public Channel run() throws IOException {
                    return System.inheritedChannel();
                }
            });
            if (obj != null && obj instanceof ServerSocketChannel) {
                AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                    @Override
                    public Void run() throws IOException {
                        System.setErr(new PrintStream(new FileOutputStream(Files.createTempFile("rmid-err", null, new FileAttribute[0]).toFile())));
                        return null;
                    }
                });
                ServerSocket socket = ((ServerSocketChannel)obj).socket();
                n2 = socket.getLocalPort();
                activationServerSocketFactory = new ActivationServerSocketFactory(socket);
                System.err.println(new Date());
                System.err.println(getTextResource("rmid.inherited.channel.info") + ": " + obj);
            }
            String s = null;
            ArrayList o = new ArrayList();
            for (int i = 0; i < array.length; ++i) {
                if (array[i].equals("-port")) {
                    if (activationServerSocketFactory != null) {
                        bomb(getTextResource("rmid.syntax.port.badarg"));
                    }
                    if (i + 1 < array.length) {
                        try {
                            n2 = Integer.parseInt(array[++i]);
                        }
                        catch (NumberFormatException ex) {
                            bomb(getTextResource("rmid.syntax.port.badnumber"));
                        }
                    }
                    else {
                        bomb(getTextResource("rmid.syntax.port.missing"));
                    }
                }
                else if (array[i].equals("-log")) {
                    if (i + 1 < array.length) {
                        s = array[++i];
                    }
                    else {
                        bomb(getTextResource("rmid.syntax.log.missing"));
                    }
                }
                else if (array[i].equals("-stop")) {
                    n = true;
                }
                else if (array[i].startsWith("-C")) {
                    o.add(array[i].substring(2));
                }
                else {
                    bomb(MessageFormat.format(getTextResource("rmid.syntax.illegal.option"), new Object[] { array[i] }));
                }
            }
            if (s == null) {
                if (activationServerSocketFactory != null) {
                    bomb(getTextResource("rmid.syntax.log.required"));
                }
                else {
                    s = "log";
                }
            }
            Activation.debugExec = (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.rmi.server.activation.debugExec"));
            String name = (String)AccessController.doPrivileged(new GetPropertyAction("sun.rmi.activation.execPolicy", null));
            if (name == null) {
                if (n == 0) {
                    Activation.DefaultExecPolicy.checkConfiguration();
                }
                name = "default";
            }
            if (!name.equals("none")) {
                if (name.equals("") || name.equals("default")) {
                    name = Activation.DefaultExecPolicy.class.getName();
                }
                try {
                    Class<Object> rmiClass = getRMIClass(name);
                    Activation.execPolicy = rmiClass.newInstance();
                    Activation.execPolicyMethod = rmiClass.getMethod("checkExecCommand", new Class[] { ActivationGroupDesc.class, String[].class });
                }
                catch (Exception ex2) {
                    if (Activation.debugExec) {
                        System.err.println(getTextResource("rmid.exec.policy.exception"));
                        ex2.printStackTrace();
                    }
                    bomb(getTextResource("rmid.exec.policy.invalid"));
                }
            }
            if (n != 0) {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        System.setProperty("java.rmi.activation.port", Integer.toString(n2));
                        return null;
                    }
                });
                ActivationGroup.getSystem().shutdown();
                System.exit(0);
            }
            startActivation(n2, activationServerSocketFactory, s, (String[])o.toArray(new String[o.size()]));
            while (true) {
                while (true) {
                    try {
                        Thread.sleep(Long.MAX_VALUE);
                        continue;
                    }
                    catch (InterruptedException ex3) {}
                }
            }
        }
        catch (Exception ex4) {
            System.err.println(MessageFormat.format(getTextResource("rmid.unexpected.exception"), new Object[] { ex4 }));
            ex4.printStackTrace();
            System.exit(1);
            return;
        }
    }
    
    private static String getTextResource(String str) {
        if (Activation.resources == null) {
            try {
                Activation.resources = ResourceBundle.getBundle("sun.rmi.server.resources.rmid");
            }
            catch (MissingResourceException ex) {}
            if (Activation.resources == null) {
                return "[missing resource file: " + str + "]";
            }
        }
        String string = null;
        try {
            string = Activation.resources.getString(str);
        }
        catch (MissingResourceException ex2) {}
        if (string == null) {
            return "[missing resource: " + str + "]";
        }
        return string;
    }
    
    private static Class<?> getRMIClass(String name) throws Exception {
        return RMIClassLoader.loadClass(name);
    }
    
    private synchronized String Pstartgroup() throws ActivationException {
        while (true) {
            this.checkShutdown();
            if (this.groupSemaphore > 0) {
                break;
            }
            try {
                this.wait();
            }
            catch (InterruptedException ex) {}
        }
        --this.groupSemaphore;
        return "Group-" + this.groupCounter++;
    }
    
    private synchronized void Vstartgroup() {
        ++this.groupSemaphore;
        this.notifyAll();
    }
    
    static void access$000(Activation activation) throws ActivationException {
        activation.checkShutdown();
    }
    
    static Activation.GroupEntry access$100(Activation activation, ActivationID activationID) throws UnknownObjectException {
        return activation.getGroupEntry(activationID);
    }
    
    static Activation.GroupEntry access$200(Activation activation, ActivationGroupID activationGroupID) throws UnknownGroupException {
        return activation.getGroupEntry(activationGroupID);
    }
    
    static void access$400(Activation activation, ActivationGroupDesc activationGroupDesc, String[] array) throws SecurityException, ActivationException {
        activation.checkArgs(activationGroupDesc, array);
    }
    
    static void access$700(Activation activation, Activation.LogRecord logRecord) throws ActivationException {
        activation.addLogRecord(logRecord);
    }
    
    static Activation.GroupEntry access$900(Activation activation, ActivationGroupID activationGroupID) throws UnknownGroupException {
        return activation.removeGroupEntry(activationGroupID);
    }
    
    static ActivationGroupID access$1000(Activation activation, ActivationID activationID) throws UnknownObjectException {
        return activation.getGroupID(activationID);
    }
    
    static void access$1400(Remote remote) {
        unexport(remote);
    }
    
    static String access$1800(String s) {
        return getTextResource(s);
    }
    
    static String access$2100(Activation activation) throws ActivationException {
        return activation.Pstartgroup();
    }
    
    static String[] access$2200(Activation activation, ActivationGroupDesc activationGroupDesc) {
        return activation.activationArgs(activationGroupDesc);
    }
    
    static void access$2500(Activation activation) {
        activation.Vstartgroup();
    }
    
    Activation(Activation.1 privilegedAction) {
        this();
    }
    
    static {
        groupTimeout = (long)getInt("sun.rmi.activation.groupTimeout", 60000);
        snapshotInterval = getInt("sun.rmi.activation.snapshotInterval", 200);
        execTimeout = (long)getInt("sun.rmi.activation.execTimeout", 30000);
        initLock = new Object();
        Activation.initDone = false;
        Activation.resources = null;
    }
    
    private static class SystemRegistryImpl extends RegistryImpl
    {
        private static final String NAME;
        private static final long serialVersionUID = 4877330021609408794L;
        private final ActivationSystem systemStub;
        
        SystemRegistryImpl(int port, RMIClientSocketFactory csf, RMIServerSocketFactory ssf, ActivationSystem systemStub) throws RemoteException {
            super(port, csf, ssf);
            this.systemStub = systemStub;
        }
        
        @Override
        public Remote lookup(String name) throws RemoteException, NotBoundException {
            if (name.equals(Activation.SystemRegistryImpl.NAME)) {
                return this.systemStub;
            }
            return super.lookup(name);
        }
        
        @Override
        public String[] list() throws RemoteException {
            String[] list = super.list();
            int length = list.length;
            String[] array = new String[length + 1];
            if (length > 0) {
                System.arraycopy(list, 0, array, 0, length);
            }
            array[length] = Activation.SystemRegistryImpl.NAME;
            return array;
        }
        
        @Override
        public void bind(String name, Remote obj) throws RemoteException, AlreadyBoundException, AccessException {
            if (name.equals(Activation.SystemRegistryImpl.NAME)) {
                throw new AccessException("binding ActivationSystem is disallowed");
            }
            RegistryImpl.checkAccess("ActivationSystem.bind");
            super.bind(name, obj);
            return;
        }
        
        @Override
        public void unbind(String name) throws RemoteException, NotBoundException, AccessException {
            if (name.equals(Activation.SystemRegistryImpl.NAME)) {
                throw new AccessException("unbinding ActivationSystem is disallowed");
            }
            RegistryImpl.checkAccess("ActivationSystem.unbind");
            super.unbind(name);
            return;
        }
        
        @Override
        public void rebind(String name, Remote obj) throws RemoteException, AccessException {
            if (name.equals(Activation.SystemRegistryImpl.NAME)) {
                throw new AccessException("binding ActivationSystem is disallowed");
            }
            RegistryImpl.checkAccess("ActivationSystem.rebind");
            super.rebind(name, obj);
            return;
        }
        
        static {
            NAME = ActivationSystem.class.getName();
        }
    }
    
    class ActivatorImpl extends RemoteServer implements Activator
    {
        private static final long serialVersionUID = -3654244726254566136L;
        
        ActivatorImpl(int port, RMIServerSocketFactory ssf) throws RemoteException {
            (this.ref = new UnicastServerRef(new LiveRef(new ObjID(1), port, null, ssf))).exportObject(this, null, false);
        }
        
        public MarshalledObject<? extends Remote> activate(ActivationID activationID, boolean b) throws ActivationException, UnknownObjectException, RemoteException {
            Activation.access$000(Activation.this);
            return Activation.access$100(Activation.this, activationID).activate(activationID, b);
        }
    }
    
    class ActivationMonitorImpl extends UnicastRemoteObject implements ActivationMonitor
    {
        private static final long serialVersionUID = -6214940464757948867L;
        
        ActivationMonitorImpl(int port, RMIServerSocketFactory ssf) throws RemoteException {
            super(port, null, ssf);
        }
        
        public void inactiveObject(ActivationID activationID) throws UnknownObjectException, RemoteException {
            try {
                Activation.access$000(Activation.this);
            }
            catch (ActivationException ex) {
                return;
            }
            RegistryImpl.checkAccess("Activator.inactiveObject");
            Activation.access$100(Activation.this, activationID).inactiveObject(activationID);
        }
        
        public void activeObject(ActivationID activationID, MarshalledObject<? extends Remote> marshalledObject) throws UnknownObjectException, RemoteException {
            try {
                Activation.access$000(Activation.this);
            }
            catch (ActivationException ex) {
                return;
            }
            RegistryImpl.checkAccess("ActivationSystem.activeObject");
            Activation.access$100(Activation.this, activationID).activeObject(activationID, marshalledObject);
        }
        
        public void inactiveGroup(ActivationGroupID activationGroupID, long n) throws UnknownGroupException, RemoteException {
            try {
                Activation.access$000(Activation.this);
            }
            catch (ActivationException ex) {
                return;
            }
            RegistryImpl.checkAccess("ActivationMonitor.inactiveGroup");
            Activation.access$200(Activation.this, activationGroupID).inactiveGroup(n, false);
        }
    }
    
    static class SameHostOnlyServerRef extends UnicastServerRef
    {
        private static final long serialVersionUID = 1234L;
        private String accessKind;
        
        SameHostOnlyServerRef(LiveRef liveRef, String accessKind) {
            super(liveRef);
            this.accessKind = accessKind;
        }
        
        @Override
        protected void unmarshalCustomCallData(ObjectInput objectInput) throws IOException, ClassNotFoundException {
            RegistryImpl.checkAccess(this.accessKind);
            super.unmarshalCustomCallData(objectInput);
        }
    }
    
    class ActivationSystemImpl extends RemoteServer implements ActivationSystem
    {
        private static final long serialVersionUID = 9100152600327688967L;
        
        ActivationSystemImpl(int port, RMIServerSocketFactory ssf) throws RemoteException {
            (this.ref = new Activation.SameHostOnlyServerRef(new LiveRef(new ObjID(4), port, null, ssf), "ActivationSystem.nonLocalAccess")).exportObject(this, null);
        }
        
        public ActivationID registerObject(ActivationDesc activationDesc) throws ActivationException, UnknownGroupException, RemoteException {
            Activation.access$000(Activation.this);
            ActivationGroupID groupID = activationDesc.getGroupID();
            ActivationID activationID = new ActivationID(Activation.this.activatorStub);
            Activation.access$200(Activation.this, groupID).registerObject(activationID, activationDesc, true);
            return activationID;
        }
        
        public void unregisterObject(ActivationID activationID) throws ActivationException, UnknownObjectException, RemoteException {
            Activation.access$000(Activation.this);
            Activation.access$100(Activation.this, activationID).unregisterObject(activationID, true);
        }
        
        public ActivationGroupID registerGroup(ActivationGroupDesc activationGroupDesc) throws ActivationException, RemoteException {
            Thread.dumpStack();
            Activation.access$000(Activation.this);
            Activation.access$400(Activation.this, activationGroupDesc, (String[])null);
            ActivationGroupID activationGroupID = new ActivationGroupID(Activation.this.systemStub);
            Activation.this.groupTable.put(activationGroupID, new Activation.GroupEntry(activationGroupID, activationGroupDesc));
            Activation.access$700(Activation.this, new Activation.LogRegisterGroup(activationGroupID, activationGroupDesc));
            return activationGroupID;
        }
        
        public ActivationMonitor activeGroup(ActivationGroupID activationGroupID, ActivationInstantiator activationInstantiator, long n) throws ActivationException, UnknownGroupException, RemoteException {
            Activation.access$000(Activation.this);
            Activation.access$200(Activation.this, activationGroupID).activeGroup(activationInstantiator, n);
            return Activation.this.monitor;
        }
        
        public void unregisterGroup(ActivationGroupID activationGroupID) throws ActivationException, UnknownGroupException, RemoteException {
            Activation.access$000(Activation.this);
            Activation.access$900(Activation.this, activationGroupID).unregisterGroup(true);
        }
        
        public ActivationDesc setActivationDesc(ActivationID activationID, ActivationDesc activationDesc) throws ActivationException, UnknownObjectException, RemoteException {
            Activation.access$000(Activation.this);
            if (!Activation.access$1000(Activation.this, activationID).equals(activationDesc.getGroupID())) {
                throw new ActivationException("ActivationDesc contains wrong group");
            }
            return Activation.access$100(Activation.this, activationID).setActivationDesc(activationID, activationDesc, true);
        }
        
        public ActivationGroupDesc setActivationGroupDesc(ActivationGroupID activationGroupID, ActivationGroupDesc activationGroupDesc) throws ActivationException, UnknownGroupException, RemoteException {
            Activation.access$000(Activation.this);
            Activation.access$400(Activation.this, activationGroupDesc, (String[])null);
            return Activation.access$200(Activation.this, activationGroupID).setActivationGroupDesc(activationGroupID, activationGroupDesc, true);
        }
        
        public ActivationDesc getActivationDesc(ActivationID activationID) throws ActivationException, UnknownObjectException, RemoteException {
            Activation.access$000(Activation.this);
            return Activation.access$100(Activation.this, activationID).getActivationDesc(activationID);
        }
        
        public ActivationGroupDesc getActivationGroupDesc(ActivationGroupID activationGroupID) throws ActivationException, UnknownGroupException, RemoteException {
            Activation.access$000(Activation.this);
            return Activation.access$200(Activation.this, activationGroupID).desc;
        }
        
        public void shutdown() throws AccessException {
            Object access$1100 = Activation.this.startupLock;
            if (access$1100 != null) {
                synchronized (access$1100) {}
            }
            synchronized (Activation.this) {
                if (!Activation.this.shuttingDown) {
                    Activation.this.shuttingDown = true;
                    new Activation.Shutdown().start();
                }
            }
        }
    }
    
    private class Shutdown extends Thread
    {
        Shutdown() {
            super("rmid Shutdown");
        }
        
        @Override
        public void run() {
            try {
                Activation.access$1400(Activation.this.activator);
                Activation.access$1400(Activation.this.system);
                Iterator<Object> iterator = Activation.this.groupTable.values().iterator();
                while (iterator.hasNext()) {
                    ((Activation.GroupEntry)iterator.next()).shutdown();
                }
                Runtime.getRuntime().removeShutdownHook(Activation.this.shutdownHook);
                Activation.access$1400(Activation.this.monitor);
                try {
                    synchronized (Activation.this.log) {
                        Activation.this.log.close();
                    }
                }
                catch (IOException ex) {}
            }
            finally {
                System.err.println(Activation.access$1800("rmid.daemon.shutdown"));
                System.exit(0);
            }
        }
    }
    
    private class ShutdownHook extends Thread
    {
        ShutdownHook() {
            super("rmid ShutdownHook");
        }
        
        @Override
        public void run() {
            synchronized (Activation.this) {
                Activation.this.shuttingDown = true;
            }
            Iterator<Object> iterator = Activation.this.groupTable.values().iterator();
            while (iterator.hasNext()) {
                ((Activation.GroupEntry)iterator.next()).shutdownFast();
            }
        }
    }
    
    private class GroupEntry implements Serializable
    {
        private static final long serialVersionUID = 7222464070032993304L;
        private static final int MAX_TRIES = 2;
        private static final int NORMAL = 0;
        private static final int CREATING = 1;
        private static final int TERMINATE = 2;
        private static final int TERMINATING = 3;
        ActivationGroupDesc desc;
        ActivationGroupID groupID;
        long incarnation;
        Map<ActivationID, Activation.ObjectEntry> objects;
        Set<ActivationID> restartSet;
        transient ActivationInstantiator group;
        transient int status;
        transient long waitTime;
        transient String groupName;
        transient Process child;
        transient boolean removed;
        transient Activation.GroupEntry.Watchdog watchdog;
        
        GroupEntry(ActivationGroupID groupID, ActivationGroupDesc desc) {
            this.desc = null;
            this.groupID = null;
            this.incarnation = 0L;
            this.objects = new HashMap();
            this.restartSet = new HashSet();
            this.group = null;
            this.status = 0;
            this.waitTime = 0L;
            this.groupName = null;
            this.child = null;
            this.removed = false;
            this.watchdog = null;
            this.groupID = groupID;
            this.desc = desc;
        }
        
        void restartServices() {
            Iterator<Object> iterator = null;
            synchronized (this) {
                if (this.restartSet.isEmpty()) {
                    return;
                }
                iterator = new HashSet(this.restartSet).iterator();
            }
            while (iterator.hasNext()) {
                ActivationID activationID = (ActivationID)iterator.next();
                try {
                    this.activate(activationID, true);
                }
                catch (Exception ex) {
                    if (Activation.this.shuttingDown) {
                        return;
                    }
                    System.err.println(Activation.access$1800("rmid.restart.service.warning"));
                    ex.printStackTrace();
                }
            }
        }
        
        synchronized void activeGroup(ActivationInstantiator activationInstantiator, long n) throws ActivationException, UnknownGroupException {
            if (this.incarnation != n) {
                throw new ActivationException("invalid incarnation");
            }
            if (this.group != null) {
                if (this.group.equals(activationInstantiator)) {
                    return;
                }
                throw new ActivationException("group already active");
            }
            else {
                if (this.child != null && this.status != 1) {
                    throw new ActivationException("group not being created");
                }
                this.group = activationInstantiator;
                this.status = 0;
                this.notifyAll();
                return;
            }
        }
        
        private void checkRemoved() throws UnknownGroupException {
            if (this.removed) {
                throw new UnknownGroupException("group removed");
            }
            return;
        }
        
        private Activation.ObjectEntry getObjectEntry(ActivationID activationID) throws UnknownObjectException {
            if (this.removed) {
                throw new UnknownObjectException("object's group removed");
            }
            Activation.ObjectEntry objectEntry = (Activation.ObjectEntry)this.objects.get(activationID);
            if (objectEntry == null) {
                throw new UnknownObjectException("object unknown");
            }
            return objectEntry;
        }
        
        synchronized void registerObject(ActivationID activationID, ActivationDesc activationDesc, boolean b) throws UnknownGroupException, ActivationException {
            this.checkRemoved();
            this.objects.put(activationID, new ObjectEntry(activationDesc));
            if (activationDesc.getRestartMode()) {
                this.restartSet.add(activationID);
            }
            Activation.this.idTable.put(activationID, this.groupID);
            if (b) {
                Activation.access$700(Activation.this, new Activation.LogRegisterObject(activationID, activationDesc));
            }
        }
        
        synchronized void unregisterObject(ActivationID activationID, boolean b) throws UnknownGroupException, ActivationException {
            Activation.ObjectEntry objectEntry = this.getObjectEntry(activationID);
            objectEntry.removed = true;
            this.objects.remove(activationID);
            if (objectEntry.desc.getRestartMode()) {
                this.restartSet.remove(activationID);
            }
            Activation.this.idTable.remove(activationID);
            if (b) {
                Activation.access$700(Activation.this, new LogUnregisterObject(activationID));
            }
        }
        
        synchronized void unregisterGroup(boolean b) throws UnknownGroupException, ActivationException {
            this.checkRemoved();
            this.removed = true;
            for (Map.Entry entry : this.objects.entrySet()) {
                Activation.this.idTable.remove((Object)entry.getKey());
                ((Activation.ObjectEntry)entry.getValue()).removed = true;
            }
            this.objects.clear();
            this.restartSet.clear();
            this.reset();
            this.childGone();
            if (b) {
                Activation.access$700(Activation.this, new Activation.LogUnregisterGroup(this.groupID));
            }
        }
        
        synchronized ActivationDesc setActivationDesc(ActivationID activationID, ActivationDesc desc, boolean b) throws UnknownObjectException, UnknownGroupException, ActivationException {
            Activation.ObjectEntry objectEntry = this.getObjectEntry(activationID);
            ActivationDesc desc2 = objectEntry.desc;
            objectEntry.desc = desc;
            if (desc.getRestartMode()) {
                this.restartSet.add(activationID);
            }
            else {
                this.restartSet.remove(activationID);
            }
            if (b) {
                Activation.access$700(Activation.this, new Activation.LogUpdateDesc(activationID, desc));
                return desc2;
            }
            return desc2;
        }
        
        synchronized ActivationDesc getActivationDesc(ActivationID activationID) throws UnknownObjectException, UnknownGroupException {
            return this.getObjectEntry(activationID).desc;
        }
        
        synchronized ActivationGroupDesc setActivationGroupDesc(ActivationGroupID activationGroupID, ActivationGroupDesc desc, boolean b) throws UnknownGroupException, ActivationException {
            this.checkRemoved();
            ActivationGroupDesc desc2 = this.desc;
            this.desc = desc;
            if (b) {
                Activation.access$700(Activation.this, new Activation.LogUpdateGroupDesc(activationGroupID, desc));
                return desc2;
            }
            return desc2;
        }
        
        synchronized void inactiveGroup(long n, boolean b) throws UnknownGroupException {
            this.checkRemoved();
            if (this.incarnation != n) {
                throw new UnknownGroupException("invalid incarnation");
            }
            this.reset();
            if (b) {
                this.terminate();
            }
            else if (this.child != null && this.status == 0) {
                this.status = 2;
                this.watchdog.noRestart();
            }
            return;
        }
        
        synchronized void activeObject(ActivationID activationID, MarshalledObject<? extends Remote> stub) throws UnknownObjectException {
            this.getObjectEntry(activationID).stub = stub;
        }
        
        synchronized void inactiveObject(ActivationID activationID) throws UnknownObjectException {
            this.getObjectEntry(activationID).reset();
        }
        
        private synchronized void reset() {
            this.group = null;
            Iterator<Activation.ObjectEntry> iterator = this.objects.values().iterator();
            while (iterator.hasNext()) {
                ((Activation.ObjectEntry)iterator.next()).reset();
            }
        }
        
        private void childGone() {
            if (this.child != null) {
                this.child = null;
                this.watchdog.dispose();
                this.watchdog = null;
                this.status = 0;
                this.notifyAll();
            }
        }
        
        private void terminate() {
            if (this.child != null && this.status != 3) {
                this.child.destroy();
                this.status = 3;
                this.waitTime = System.currentTimeMillis() + Activation.groupTimeout;
                this.notifyAll();
            }
        }
        
        private void await() {
            while (true) {
                switch (this.status) {
                    case 0: {
                        return;
                    }
                    case 2: {
                        this.terminate();
                    }
                    case 3: {
                        try {
                            this.child.exitValue();
                        }
                        catch (IllegalThreadStateException ex) {
                            long currentTimeMillis = System.currentTimeMillis();
                            if (this.waitTime > currentTimeMillis) {
                                try {
                                    this.wait(this.waitTime - currentTimeMillis);
                                }
                                catch (InterruptedException ex2) {}
                                continue;
                            }
                        }
                        this.childGone();
                        return;
                    }
                    case 1: {
                        try {
                            this.wait();
                        }
                        catch (InterruptedException ex3) {}
                        continue;
                    }
                }
            }
        }
        
        void shutdownFast() {
            Process child = this.child;
            if (child != null) {
                child.destroy();
            }
        }
        
        synchronized void shutdown() {
            this.reset();
            this.terminate();
            this.await();
        }
        
        MarshalledObject<? extends Remote> activate(ActivationID activationID, boolean b) throws ActivationException {
            Object o = null;
            for (int i = 2; i > 0; --i) {
                Activation.ObjectEntry objectEntry;
                ActivationInstantiator instantiator;
                long incarnation;
                synchronized (this) {
                    objectEntry = this.getObjectEntry(activationID);
                    if (!b && objectEntry.stub != null) {
                        return objectEntry.stub;
                    }
                    instantiator = this.getInstantiator(this.groupID);
                    incarnation = this.incarnation;
                }
                boolean n = false;
                boolean n2 = false;
                try {
                    return objectEntry.activate(activationID, b, instantiator);
                }
                catch (NoSuchObjectException ex) {
                    n = true;
                    o = ex;
                }
                catch (ConnectException ex2) {
                    n = true;
                    n2 = true;
                    o = ex2;
                }
                catch (ConnectIOException ex3) {
                    n = true;
                    n2 = true;
                    o = ex3;
                }
                catch (InactiveGroupException ex4) {
                    n = true;
                    o = ex4;
                }
                catch (RemoteException ex5) {
                    if (o == null) {
                        o = ex5;
                    }
                }
                if (!(n == 0)) {
                    try {
                        System.err.println(MessageFormat.format(Activation.access$1800("rmid.group.inactive"), new Object[] { o.toString() }));
                        o.printStackTrace();
                        Activation.access$200(Activation.this, this.groupID).inactiveGroup(incarnation, (boolean)n2);
                    }
                    catch (UnknownGroupException ex6) {}
                }
            }
            throw new ActivationException("object activation failed after 2 tries", o);
        }
        
        private ActivationInstantiator getInstantiator(ActivationGroupID obj) throws ActivationException {
            assert !(!Thread.holdsLock(this));
            this.await();
            if (this.group != null) {
                return this.group;
            }
            this.checkRemoved();
            boolean n = false;
            try {
                this.groupName = Activation.access$2100(Activation.this);
                n = true;
                String[] access$2200 = Activation.access$2200(Activation.this, this.desc);
                Activation.access$400(Activation.this, this.desc, access$2200);
                if (Activation.debugExec) {
                    StringBuffer sb = new StringBuffer(access$2200[0]);
                    for (int i = 1; i < access$2200.length; ++i) {
                        sb.append(' ');
                        sb.append(access$2200[i]);
                    }
                    System.err.println(MessageFormat.format(Activation.access$1800("rmid.exec.command"), new Object[] { sb.toString() }));
                }
                try {
                    this.child = Runtime.getRuntime().exec(access$2200);
                    this.status = 1;
                    ++this.incarnation;
                    (this.watchdog = new Activation.GroupEntry.Watchdog()).start();
                    Activation.access$700(Activation.this, new Activation.LogGroupIncarnation(obj, this.incarnation));
                    PipeWriter.plugTogetherPair(this.child.getInputStream(), System.out, this.child.getErrorStream(), System.err);
                    try (final Object o = new MarshalOutputStream(this.child.getOutputStream())) {
                        o.writeObject(obj);
                        o.writeObject(this.desc);
                        o.writeLong(this.incarnation);
                        o.flush();
                    }
                }
                catch (IOException ex) {
                    this.terminate();
                    throw new ActivationException("unable to create activation group", ex);
                }
                try {
                    long n2 = System.currentTimeMillis();
                    long n3 = n2 + Activation.execTimeout;
                    do {
                        this.wait(n3 - n2);
                        if (this.group != null) {
                            ActivationInstantiator group = this.group;
                        }
                        else {
                            n2 = System.currentTimeMillis();
                        }
                    } while (this.status == 1 && n2 < n3);
                }
                catch (InterruptedException ex2) {}
                this.terminate();
                throw new ActivationException(this.removed ? "activation group unregistered" : "timeout creating child process");
            }
            finally {
                if (!(n == 0)) {
                    Activation.access$2500(Activation.this);
                }
            }
        }
        
        static void access$2600(Activation.GroupEntry groupEntry) {
            groupEntry.reset();
        }
        
        static void access$2700(Activation.GroupEntry groupEntry) {
            groupEntry.childGone();
        }
        
        private class Watchdog extends Thread
        {
            private final Process groupProcess;
            private final long groupIncarnation;
            private boolean canInterrupt;
            private boolean shouldQuit;
            private boolean shouldRestart;
            
            Watchdog() {
                super("WatchDog-" + Activation.GroupEntry.this.groupName + "-" + Activation.GroupEntry.this.incarnation);
                this.groupProcess = Activation.GroupEntry.this.child;
                this.groupIncarnation = Activation.GroupEntry.this.incarnation;
                this.canInterrupt = true;
                this.shouldQuit = false;
                this.setDaemon(this.shouldRestart = true);
            }
            
            @Override
            public void run() {
                if (this.shouldQuit) {
                    return;
                }
                try {
                    this.groupProcess.waitFor();
                }
                catch (InterruptedException ex) {
                    return;
                }
                int n = 0;
                synchronized (Activation.GroupEntry.this) {
                    if (this.shouldQuit) {
                        return;
                    }
                    this.canInterrupt = false;
                    interrupted();
                    if (this.groupIncarnation == Activation.GroupEntry.this.incarnation) {
                        n = (this.shouldRestart && !Activation.this.shuttingDown);
                        Activation.GroupEntry.access$2600(Activation.GroupEntry.this);
                        Activation.GroupEntry.access$2700(Activation.GroupEntry.this);
                    }
                }
                if (n != 0) {
                    Activation.GroupEntry.this.restartServices();
                }
                return;
            }
            
            void dispose() {
                this.shouldQuit = true;
                if (this.canInterrupt) {
                    this.interrupt();
                }
            }
            
            void noRestart() {
                this.shouldRestart = false;
            }
        }
    }
    
    private static class ObjectEntry implements Serializable
    {
        private static final long serialVersionUID = -5500114225321357856L;
        ActivationDesc desc;
        transient volatile MarshalledObject<? extends Remote> stub;
        transient volatile boolean removed;
        
        ObjectEntry(ActivationDesc desc) {
            this.stub = null;
            this.removed = false;
            this.desc = desc;
        }
        
        synchronized MarshalledObject<? extends Remote> activate(ActivationID activationID, boolean b, ActivationInstantiator activationInstantiator) throws RemoteException, ActivationException {
            MarshalledObject<Object> stub = this.stub;
            if (this.removed) {
                throw new UnknownObjectException("object removed");
            }
            if (!b && stub != null) {
                return stub;
            }
            return this.stub = activationInstantiator.newInstance(activationID, this.desc);
        }
        
        void reset() {
            this.stub = null;
        }
    }
    
    private static class ActLogHandler extends LogHandler
    {
        ActLogHandler() {
        }
        
        @Override
        public Object initialSnapshot() {
            return new Activation();
        }
        
        @Override
        public Object applyUpdate(Object o, Object o2) throws Exception {
            return ((Activation.LogRecord)o).apply(o2);
        }
    }
    
    private abstract static class LogRecord implements Serializable
    {
        private static final long serialVersionUID = 8395140512322687529L;
        
        abstract Object apply(Object p0) throws Exception;
    }
    
    private static class LogRegisterObject extends Activation.LogRecord
    {
        private static final long serialVersionUID = -6280336276146085143L;
        private ActivationID id;
        private ActivationDesc desc;
        
        LogRegisterObject(ActivationID id, ActivationDesc desc) {
            this.id = id;
            this.desc = desc;
        }
        
        @Override
        Object apply(Object o) {
            try {
                Activation.access$200((Activation)o, this.desc.getGroupID()).registerObject(this.id, this.desc, false);
            }
            catch (Exception ex) {
                System.err.println(MessageFormat.format(Activation.access$1800("rmid.log.recover.warning"), new Object[] { "LogRegisterObject" }));
                ex.printStackTrace();
            }
            return o;
        }
    }
    
    private static class LogUnregisterObject extends Activation.LogRecord
    {
        private static final long serialVersionUID = 6269824097396935501L;
        private ActivationID id;
        
        LogUnregisterObject(ActivationID id) {
            this.id = id;
        }
        
        @Override
        Object apply(Object o) {
            try {
                Activation.access$100((Activation)o, this.id).unregisterObject(this.id, false);
            }
            catch (Exception ex) {
                System.err.println(MessageFormat.format(Activation.access$1800("rmid.log.recover.warning"), new Object[] { "LogUnregisterObject" }));
                ex.printStackTrace();
            }
            return o;
        }
    }
    
    private static class LogRegisterGroup extends Activation.LogRecord
    {
        private static final long serialVersionUID = -1966827458515403625L;
        private ActivationGroupID id;
        private ActivationGroupDesc desc;
        
        LogRegisterGroup(ActivationGroupID id, ActivationGroupDesc desc) {
            this.id = id;
            this.desc = desc;
        }
        
        @Override
        Object apply(Object o) {
            ((Activation)o).groupTable.put(this.id, (Activation)o.new Activation.GroupEntry(this.id, this.desc));
            return o;
        }
    }
    
    private static class LogUpdateDesc extends Activation.LogRecord
    {
        private static final long serialVersionUID = 545511539051179885L;
        private ActivationID id;
        private ActivationDesc desc;
        
        LogUpdateDesc(ActivationID id, ActivationDesc desc) {
            this.id = id;
            this.desc = desc;
        }
        
        @Override
        Object apply(Object o) {
            try {
                Activation.access$100((Activation)o, this.id).setActivationDesc(this.id, this.desc, false);
            }
            catch (Exception ex) {
                System.err.println(MessageFormat.format(Activation.access$1800("rmid.log.recover.warning"), new Object[] { "LogUpdateDesc" }));
                ex.printStackTrace();
            }
            return o;
        }
    }
    
    private static class LogUpdateGroupDesc extends Activation.LogRecord
    {
        private static final long serialVersionUID = -1271300989218424337L;
        private ActivationGroupID id;
        private ActivationGroupDesc desc;
        
        LogUpdateGroupDesc(ActivationGroupID id, ActivationGroupDesc desc) {
            this.id = id;
            this.desc = desc;
        }
        
        @Override
        Object apply(Object o) {
            try {
                Activation.access$200((Activation)o, this.id).setActivationGroupDesc(this.id, this.desc, false);
            }
            catch (Exception ex) {
                System.err.println(MessageFormat.format(Activation.access$1800("rmid.log.recover.warning"), new Object[] { "LogUpdateGroupDesc" }));
                ex.printStackTrace();
            }
            return o;
        }
    }
    
    private static class LogUnregisterGroup extends Activation.LogRecord
    {
        private static final long serialVersionUID = -3356306586522147344L;
        private ActivationGroupID id;
        
        LogUnregisterGroup(ActivationGroupID id) {
            this.id = id;
        }
        
        @Override
        Object apply(Object o) {
            Activation.GroupEntry groupEntry = (Activation.GroupEntry)((Activation)o).groupTable.remove(this.id);
            try {
                groupEntry.unregisterGroup(false);
            }
            catch (Exception ex) {
                System.err.println(MessageFormat.format(Activation.access$1800("rmid.log.recover.warning"), new Object[] { "LogUnregisterGroup" }));
                ex.printStackTrace();
            }
            return o;
        }
    }
    
    private static class LogGroupIncarnation extends Activation.LogRecord
    {
        private static final long serialVersionUID = 4146872747377631897L;
        private ActivationGroupID id;
        private long inc;
        
        LogGroupIncarnation(ActivationGroupID id, long inc) {
            this.id = id;
            this.inc = inc;
        }
        
        @Override
        Object apply(Object o) {
            try {
                Activation.access$200((Activation)o, this.id).incarnation = this.inc;
            }
            catch (Exception ex) {
                System.err.println(MessageFormat.format(Activation.access$1800("rmid.log.recover.warning"), new Object[] { "LogGroupIncarnation" }));
                ex.printStackTrace();
            }
            return o;
        }
    }
    
    public static class DefaultExecPolicy
    {
        public void checkExecCommand(ActivationGroupDesc activationGroupDesc, String[] array) throws SecurityException {
            PermissionCollection execPermissions = getExecPermissions();
            Properties propertyOverrides = activationGroupDesc.getPropertyOverrides();
            if (propertyOverrides != null) {
                Enumeration<Object> propertyNames = propertyOverrides.propertyNames();
                while (propertyNames.hasMoreElements()) {
                    String str = (String)propertyNames.nextElement();
                    String property = propertyOverrides.getProperty(str);
                    String string = "-D" + str + "=" + property;
                    try {
                        checkPermission(execPermissions, new ExecOptionPermission(string));
                    }
                    catch (AccessControlException ex) {
                        if (property.equals("")) {
                            checkPermission(execPermissions, new ExecOptionPermission("-D" + str));
                        }
                        else {
                            throw ex;
                        }
                    }
                }
            }
            String className = activationGroupDesc.getClassName();
            if ((className != null && !className.equals(ActivationGroupImpl.class.getName())) || activationGroupDesc.getLocation() != null || activationGroupDesc.getData() != null) {
                throw new AccessControlException("access denied (custom group implementation not allowed)");
            }
            ActivationGroupDesc.CommandEnvironment commandEnvironment = activationGroupDesc.getCommandEnvironment();
            if (commandEnvironment != null) {
                String commandPath = commandEnvironment.getCommandPath();
                if (commandPath != null) {
                    checkPermission(execPermissions, new ExecPermission(commandPath));
                }
                String[] commandOptions = commandEnvironment.getCommandOptions();
                if (commandOptions != null) {
                    for (int length = commandOptions.length, i = 0; i < length; ++i) {
                        checkPermission(execPermissions, new ExecOptionPermission(commandOptions[i]));
                    }
                }
            }
            return;
        }
        
        static void checkConfiguration() {
            if (!(((Policy)AccessController.doPrivileged(new PrivilegedAction<Policy>() {
                @Override
                public Policy run() {
                    return Policy.getPolicy();
                }
            })) instanceof PolicyFile)) {
                return;
            }
            Enumeration<Permission> elements = getExecPermissions().elements();
            while (elements.hasMoreElements()) {
                Permission permission = (Permission)elements.nextElement();
                if (permission instanceof AllPermission || permission instanceof ExecPermission || permission instanceof ExecOptionPermission) {
                    return;
                }
                continue;
            }
            System.err.println(Activation.access$1800("rmid.exec.perms.inadequate"));
            return;
        }
        
        private static PermissionCollection getExecPermissions() {
            return (PermissionCollection)AccessController.doPrivileged(new PrivilegedAction<PermissionCollection>() {
                @Override
                public PermissionCollection run() {
                    CodeSource codesource = new CodeSource(null, (Certificate[])null);
                    Policy policy = Policy.getPolicy();
                    if (policy != null) {
                        return policy.getPermissions(codesource);
                    }
                    return new Permissions();
                }
            });
        }
        
        private static void checkPermission(PermissionCollection collection, Permission permission) throws AccessControlException {
            if (!collection.implies(permission)) {
                throw new AccessControlException("access denied " + permission.toString());
            }
            return;
        }
    }
    
    private static class ActivationServerSocketFactory implements RMIServerSocketFactory
    {
        private final ServerSocket serverSocket;
        
        ActivationServerSocketFactory(ServerSocket serverSocket) {
            this.serverSocket = serverSocket;
        }
        
        @Override
        public ServerSocket createServerSocket(int n) throws IOException {
            return new Activation.DelayedAcceptServerSocket(this.serverSocket);
        }
    }
    
    private static class DelayedAcceptServerSocket extends ServerSocket
    {
        private final ServerSocket serverSocket;
        
        DelayedAcceptServerSocket(ServerSocket serverSocket) throws IOException {
            this.serverSocket = serverSocket;
        }
        
        @Override
        public void bind(SocketAddress endpoint) throws IOException {
            this.serverSocket.bind(endpoint);
        }
        
        @Override
        public void bind(SocketAddress endpoint, int backlog) throws IOException {
            this.serverSocket.bind(endpoint, backlog);
        }
        
        @Override
        public InetAddress getInetAddress() {
            return (InetAddress)AccessController.doPrivileged(new PrivilegedAction<InetAddress>() {
                @Override
                public InetAddress run() {
                    return Activation.DelayedAcceptServerSocket.this.serverSocket.getInetAddress();
                }
            });
        }
        
        @Override
        public int getLocalPort() {
            return this.serverSocket.getLocalPort();
        }
        
        @Override
        public SocketAddress getLocalSocketAddress() {
            return (SocketAddress)AccessController.doPrivileged(new PrivilegedAction<SocketAddress>() {
                @Override
                public SocketAddress run() {
                    return Activation.DelayedAcceptServerSocket.this.serverSocket.getLocalSocketAddress();
                }
            });
        }
        
        @Override
        public Socket accept() throws IOException {
            synchronized (Activation.initLock) {
                try {
                    while (!Activation.initDone) {
                        Activation.initLock.wait();
                    }
                }
                catch (InterruptedException detailMessage) {
                    throw new AssertionError(detailMessage);
                }
            }
            return this.serverSocket.accept();
        }
        
        @Override
        public void close() throws IOException {
            this.serverSocket.close();
        }
        
        @Override
        public ServerSocketChannel getChannel() {
            return this.serverSocket.getChannel();
        }
        
        @Override
        public boolean isBound() {
            return this.serverSocket.isBound();
        }
        
        @Override
        public boolean isClosed() {
            return this.serverSocket.isClosed();
        }
        
        @Override
        public void setSoTimeout(int soTimeout) throws SocketException {
            this.serverSocket.setSoTimeout(soTimeout);
        }
        
        @Override
        public int getSoTimeout() throws IOException {
            return this.serverSocket.getSoTimeout();
        }
        
        @Override
        public void setReuseAddress(boolean reuseAddress) throws SocketException {
            this.serverSocket.setReuseAddress(reuseAddress);
        }
        
        @Override
        public boolean getReuseAddress() throws SocketException {
            return this.serverSocket.getReuseAddress();
        }
        
        @Override
        public String toString() {
            return this.serverSocket.toString();
        }
        
        @Override
        public void setReceiveBufferSize(int receiveBufferSize) throws SocketException {
            this.serverSocket.setReceiveBufferSize(receiveBufferSize);
        }
        
        @Override
        public int getReceiveBufferSize() throws SocketException {
            return this.serverSocket.getReceiveBufferSize();
        }
    }
}
